local MG = LibStub("AceAddon-3.0"):NewAddon("MetaGuild", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0", "AceTimer-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("MetaGuild");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");
local icon = LibStub("LibDBIcon-1.0");
local ldb = LibStub:GetLibrary("LibDataBroker-1.1");

local version = "0.3";

local self = MG;
local specialChar = '\28';
local colorChar = '{%s}';
local specialLinkChar = '\29';
local playerLink = "|Hplayer %s|h[%s]|h";

function MG:CreateMsgTemplate(dataFieldCount)
	return string.rep(specialChar .. "%s", dataFieldCount) .. specialChar;
end

local status = 1;

local playerPerSite = 12;
local activeSite = 0;

-- Online list
local numOnline = 0;


-- Key Binding Setup
_G["BINDING_HEADER_METAGUILD"] = "Meta Guild";
_G["BINDING_NAME_MG_SHOW"] = "Show player list.";

-- Color Test
local red = "|cffff0000";
local green = "|cff00ff00";
local blue = "|cff0000ff";


local defaults = {
	realm = {
		main = nil,
		rank = 0,
		time = 0,
		timeOffset = 0,
		slashCmd = "mg",
		motd = {},
		mission = {},
		info = {},
		tradeskillPerChar = true,
		professions = {},
		guildName = "",
		guildChannel = "",
		guildPassword = "",
		baseChannel = "METAGUILD",
		offlinePlayers = {},
		onlinePlayers = {},
		kickedPlayers = {},
		
	},
	char = {
		professions = {},
		note = "",
	},
	profile = {
		enabled = true,
		logEnabled = false,
		filterDisabled = false,
		showAchiev = true,
		chatTag = "MetaGuild",
		scale = 1.0,
		color = {
			r = 1.0,
			g = 1.0,
			b = 1.0,
		},
		
	}
}

local dataobj = ldb:NewDataObject("MetaGuild", {
    type = "data source",
	text = "MG (" .. numOnline .. ")",
	label = "MetaGuild",
    icon = "Interface\\AddOns\\MetaGuild\\MinimapIcon.tga",
    OnClick = function(clickedframe, button)
		if button == "LeftButton" then
			MetaGuild_Toggle();
		elseif button == "RightButton" then
			if LibStub("AceConfigDialog-3.0").OpenFrames["MetaGuild"] then
				LibStub("AceConfigDialog-3.0"):Close("MetaGuild")
			else
				LibStub("AceConfigDialog-3.0"):Open("MetaGuild")
			end
		end		 
    end,
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
			tooltip:AddLine("MetaGuild");
			tooltip:AddLine("Memebers Online:");
			for k, v in pairs(self.db.realm.onlinePlayers) do
				local character = self:GetCharacterInfo(k)
				if character ~= nil then 
					local colors = RAID_CLASS_COLORS[character.class];
					if(colors == nil) then
						colors = {};
						colors.r = 0.5;
						colors.g = 0.5;
						colors.b = 0.5;
					end
					tooltip:AddLine("  " .. character.name, colors.r, colors.g, colors.b);
				end
			end
			if (numOnline == 1) then
				tooltip:AddLine(numOnline .. " player");
			else
				tooltip:AddLine(numOnline .. " players");
			end
			tooltip:AddLine("Left-click to open player list", 0, 1, 0)
			tooltip:AddLine("Right-click for options menu", 0, 1, 0)
	end,
})

local options = {
	type = "group",
	name = "MetaGuild Config",
	args = {

		system = {
			type = "group",
			name = "System",
			desc = "System settings",
			order = 50,
			args = {
				enable = {
					name = L["ConfigFrame/Enable"],
					desc = L["ConfigFrame/EnableDesc"],
					type = "toggle",
					order = 55,
					width = "full",
					set = function(info,val) 
						self.db.profile.enabled = val;
						if (val == true) then 
							self:OnEnable();
							self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 2);
						else 
							self:OnDisable() 
						end 
					end,
					get = function(info) return self.db.profile.enabled; end
				},
				guildName = {
					name = L["ConfigFrame/GuildName"],
					desc = L["ConfigFrame/GuildNameDesc"],
					order = 45,
					width = "full",
					type = "input",
					set = function(info,val)
							local wasEnabled = self.db.profile.enabled;
							if(wasEnabled) then
								self.db.profile.enabled = false;
								self:OnDisable();
							end
							self.db.realm.guildName = val;
							if(wasEnabled) then
								self.db.profile.enabled = true;
								self:OnEnable();
							end							
						  end,
					get = function(info) return self.db.realm.guildName; end
				},
				guildChannel = {
					name = L["ConfigFrame/GuildChannel"],
					desc = L["ConfigFrame/GuildChannelDesc"],
					order = 46,
					width = "full",
					type = "input",
					set = function(info,val)
							local wasEnabled = self.db.profile.enabled;
							if(wasEnabled) then
								self.db.profile.enabled = false;
								self:OnDisable();
							end
							self.db.realm.guildChannel = string.upper(val);
							if(wasEnabled) then
								self.db.profile.enabled = true;
								self:OnEnable();
							end							
						  end,
					get = function(info) return self.db.realm.guildChannel; end
				},
				guildPassword = {
					name = L["ConfigFrame/GuildPassword"],
					desc = L["ConfigFrame/GuildPasswordDesc"],
					order = 47,
					width = "double",
					type = "input",
					set = function(info,val)
							local wasEnabled = self.db.profile.enabled;
							if(wasEnabled) then
								self.db.profile.enabled = false;
								self:OnDisable();
							end
							self.db.realm.guildPassword = val;
							if(wasEnabled) then
								self.db.profile.enabled = true;
								self:OnEnable();
							end							
						  end,
					get = function(info) return self.db.realm.guildPassword; end
				},			
				main = {
					name = L["ConfigFrame/MainChar"],
					desc = L["ConfigFrame/MainCharDesc"],
					type = "execute",
					order = -2,
					func = function(info) 
						self.db.realm.main = UnitName("player");
						self:SystemMessage(L["MainChanged"]);
						self:SendUpdate();
					end,
				},
				wipe = {
					name = L["ConfigFrame/Wipe"],
					desc = L["ConfigFrame/WipeDesc"],
					type = "execute",
					order = -2,
					func = function(info) 
						self.db.realm.offlinePlayers = {}
						self.db.realm.onlinePlayers = {}
						self:SendJoin()
					end,
				},
				rank = {
					name = L["ConfigFrame/Rank"],
					desc = L["ConfigFrame/RankDesc"],
					type = "range",
					order = 60,
					min = 0,
					max = 3,
					step = 1.0,
					isPercent = false,
					set = function(info, val) self.db.realm.rank = val; self:SendUpdate(); MetaGuildGuild_Update(); end,
					get = function(info) return self.db.realm.rank end,
				},
				motd = {
					name = L["ConfigFrame/Motd"],
					desc = L["ConfigFrame/MotdDesc"],
					type = "input",
					width = "full",
					order = -1;
					set = function(info, val) self.db.realm.motd.msg = val; self:SendNewMotd(val) end,
					get = function(info) return self.db.realm.motd.msg end,
				},
				tradeskillPerChar = {
					name = L["ConfigFrame/TradeskillPerChar"],
					desc = L["ConfigFrame/TradeskillPerCharDesc"],
					type = "toggle",
					width = "double",
					set = function(info, val) self.db.realm.tradeskillPerChar = val; end,
					get = function(info) return self.db.realm.tradeskillPerChar end,
				},
				showIcon = {
					name = L["ConfigFrame/ShowIcon"],
					desc = L["ConfigFrame/ShowIconDesc"],
					type = "toggle",
					set = function(info, val) MetaGuildDB.hide = not val; if(val) then icon:Show("MetaGuild"); else icon:Hide("MetaGuild"); end end,
					get = function(info) return not MetaGuildDB.hide; end,
				},
				showAchiev = {
					name = L["ConfigFrame/ShowAchiev"],
					desc = L["ConfigFrame/ShowAchievDesc"],
					type = "toggle",
					order = -3,
					set = function(info, val) self.db.profile.showAchiev = val; end,
					get = function(info) return self.db.profile.showAchiev end,
				},
				scale = {
					name = L["ConfigFrame/Scale"],
					desc = L["ConfigFrame/ScaleDesc"],
					type = "range",
					order = 60,
					min = 0.1,
					max = 2,
					step = 0.01,
					isPercent = true,
					set = function(info, val) self.db.profile.scale = val; _G["MetaGuildFrame"]:SetScale(val);  end,
					get = function(info) return self.db.profile.scale end,
				},
			},
		},
		chat = {
			type = "group",
			name = "Chat",
			args = {
				chatColor = {
					name = L["ConfigFrame/ChatColor"],
					desc = L["ConfigFrame/ChatColorDesc"],
					type = "color",
					order = 65,
					set = function(info, r, g, b, a)
						local channel = GetChannelName(self.db.realm.guildChannel);
						if(channel > 0) then
							ChangeChatColor("CHANNEL" .. GetChannelName(self.db.realm.guildChannel), r, g, b); 
						end
						self.db.profile.color.r = r; 
						self.db.profile.color.g = g; 
						self.db.profile.color.b = b;
						--self:SystemMessage(L["ChatColorChanged"]);	-- Spamms the chat
					end,
					get = function(info) return self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b; end,
				},
				chatTag = {
					name = L["ConfigFrame/ChatTag"],
					desc = L["ConfigFrame/ChatTagDesc"],
					type = "input",
					order = 70,
					set = function(info, val)
						self.db.profile.chatTag = val;
					end,
					get = function(info)
						return self.db.profile.chatTag;
					end,
				},
			},
		},
		debug = {
			type = "group",
			name = "Debug",
			desc = "Guild Debug Controls",
			args = {
				disableFilter = {
					name = L["ConfigFrame/FilterDisabled"],
					desc = L["ConfigFrame/FilterDisabledDesc"],
					type = "toggle",
					order = 55,
					width = "full",
					set = function(info,val) 
						self.db.profile.filterDisabled = val;
					end,
					get = function(info) return self.db.profile.filterDisabled; end
				},
				join = {
					name = L["ConfigFrame/SendJoin"],
					desc = L["ConfigFrame/SendJoinDesc"],
					type = "execute",
					order = -2,
					func = function(info) 
						self:SendJoin();
					end,
				},
				update = {
					name = L["ConfigFrame/SendUpdate"],
					desc = L["ConfigFrame/SendUpdateDesc"],
					type = "execute",
					order = -2,
					func = function(info) 
						self:SendUpdate();
					end,
				},
				getTime = {
					name = L["ConfigFrame/GetTime"],
					desc = L["ConfigFrame/GetTimeDesc"],
					type = "execute",
					order = 0,
					func = function(info)
						self.db.realm.time = self:GetTimeCode()
						self:Print("Time: "..self.db.realm.time);
					end,
				},
				getTimeOffset = {
					name = "GetTimeOffset",
					desc = "This will return the time offset since you pressed the Get Time Button.",
					type = "execute",
					order = 0,
					func = function(info)
						self:Print("Time Offset: "..self:GetTimeCodeOffset(self.db.realm.time));
					end,
				},
				printTime = {
					name = "Print Time",
					desc = "This will print the current time in text form.",
					type = "execute",
					order = 1,
					func = function(info)
						local lastOnDate = self:TimeSinceLastTimeCode(self.db.realm.time)
						self:Print("Time Offset: "..self:displayTimeSinceLastOn(lastOnDate, 0));
					end,
				},				
				
				
			},
		},
		log = {
			type = "group",
			name = "Log",
			desc = "Guild Log Controls",
			args = {
				enable = {
					name = L["ConfigFrame/EnableLog"],
					desc = L["ConfigFrame/EnableLogDesc"],
					type = "toggle",
					order = 55,
					width = "full",
					set = function(info,val) 
						self.db.profile.logEnabled = val;
						if (val == true) then
							MetaGuildFrameInfoTabFrameViewLogButton:Enable()
							--self:OnEnable();
							--self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 2);
						else 
							MetaGuildFrameInfoTabFrameViewLogButton:Disable()
							--self:OnDisable() 
						end 
					end,
					get = function(info) return self.db.profile.logEnabled; end
				},
			},
		},
		show = {
			name = L["ConfigFrame/ShowPlayerlist"],
			desc = L["ConfigFrame/ShowPlayerlistDesc"],
			guiHidden  = true,
			type = "execute",
			func = function(info) MetaGuild_OnlineFrame:Show(); end,
		},
	},
	
}

function MG:InitLocalization()
	--> This will set the localized text of the GUI <--
	
	MetaGuildFrameGuildNameLabel:SetText(self.db.realm.guildName)
	MetaGuildFrameMembersCountLabel:SetText(L["Main/MembersOnlineText"])
	
	--> GUI Tabs  <--
	MetaGuildFrameTab1:SetText(L["Main/Tab_Guild"])
	MetaGuildFrameTab2:SetText(L["Main/Tab_Roster"])
	MetaGuildFrameTab3:SetText(L["Main/Tab_News"])
	MetaGuildFrameTab4:SetText(L["Main/Tab_Info"])
--	MetaGuildFrameTab5:SetText(L["Main/Tab_Debug"])
end

function MG:OnInitialize()
	-- do init tasks here, like loading the Saved Variables,
	-- or setting up slash commands.
	self.db = LibStub("AceDB-3.0"):New("MetaGuildDB", defaults, true);
	if(self.db.realm.main == nil) then
		self.db.realm.main = UnitName("player");
	end
	if(self.db.realm.rank == nil) then
		self.db.realm.rank = 0
	end
	
	if(self.db.realm.timeOffset == nil or self.db.realm.timeOffset == "" or self.db.realm.rank >= 3) then
		self.db.realm.timeOffset = 0
	end
	
	
	
	self.db.RegisterCallback(self, "OnNewProfile", "LoadedNewProfile");
	self.db.RegisterCallback(self, "OnProfileChanged", "LoadedNewProfile");
	self.db.RegisterCallback(self, "OnProfileCopied", "LoadedNewProfile");
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileReset");
	self.db.RegisterCallback(self, "OnProfileShutdown", "LeaveProfile");
	
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	icon:Register("MetaGuild", dataobj, MetaGuildDB);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MetaGuild", options, {"MetaGuild", "MG", "Meta"});
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("MetaGuild", "MetaGuild");
	self:RegisterChatCommand("metaguild", "SlashCommand")
	self:RegisterChatCommand("mg", "SlashCommand")
	self:RegisterChatCommand("meta", "SlashCommand")
	
	tinsert(UISpecialFrames,"MetaGuild_OnlineFrame");
	
	self:InitPopupDialogs()
	
	
end

function PanelTemplates_UpdateTabsTest(frame)
	--Not sure this is being used. Kill it if needed.
	if ( frame.selectedTab ) then
		local tab;
		for i=1, frame.numTabs, 1 do
			tab = _G[frame:GetName().."Tab"..i];
			print(tab)
			if ( tab.isDisabled ) then
				PanelTemplates_SetDisabledTabState(tab);
			elseif ( i == frame.selectedTab ) then
				PanelTemplates_SelectTab(tab);
			else
				PanelTemplates_DeselectTab(tab);
			end
		end
	end
end


function MG:OnEnable()
	-- Do more initialization here, that really enables the use of your addon.
	-- Register Events, Hook functions, Create Frames, Get information from 
	-- the game that wasn't available in OnInitialize
	if (self.db.profile.enabled == false) then
		return;
	end
	
	-- Wipe the Online Database
	self.db.realm.onlinePlayers = {}
	
	-- Events
	self:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE")
	self:RegisterEvent("CHAT_MSG_CHANNEL_JOIN")
	self:RegisterEvent("CHAT_MSG_CHANNEL_LEAVE")
	self:RegisterEvent("CHAT_MSG_CHANNEL")
	self:RegisterEvent("CHAT_MSG_SYSTEM")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("ACHIEVEMENT_EARNED")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("CHAT_MSG_CHANNEL_LIST")
	
	
	self:JoinMetaGuildChannels()

	self:RegisterComm(string.upper(self.db.realm.baseChannel))
	self:Print("Just setup the comm for "..self.db.realm.baseChannel)
--	if self.db.realm.guildChannel then 
		--self:RegisterComm(string.upper(self.db.realm.guildChannel))
--	end
	-- Hooks
	if not self:IsHooked("SendChatMessage") then
		self:RawHook("SendChatMessage", true);
	end
	
	if not self:IsHooked("ChatEdit_UpdateHeader") then
		self:RawHook("ChatEdit_UpdateHeader", true);
	end
	
	--[[if not self:IsHooked("PlayerNameAutocomplete") then
		MG:RawHook("PlayerNameAutocomplete", true);
	end]]--
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", MetaGuildChatFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL_NOTICE_USER", MetaGuildChatRemover);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL_LIST", MetaGuildChatRemover);	


	-->	Set all the GUI Names from the Localization array	<--
	self:InitLocalization()

	
	if(GetChannelName(self.db.realm.guildChannel) > 0) then
		self:OnChannelJoined();
	else
		-- Don't join channel yet, because it will discard messages. It will be done after another channel is joined.
		-- self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 3);
	end
	
	_G["MetaGuildFrame"]:SetScale(self.db.profile.scale);
	

	-->  Setup the Tabs  <--
	PanelTemplates_SetNumTabs(MetaGuildFrame, 4);
 	PanelTemplates_SetTab(MetaGuildFrame, 2);      -- 1 because we want tab 1 selected.
 	MetaGuildFrameGuildTabFrame:Hide();  -- Hide all other pages (in this case only one).
 	MetaGuildFrameRosterTabFrame:Show();  -- Show page 1.
 	MetaGuildFrameNewsTabFrame:Hide();  -- Hide all other pages (in this case only one).
	MetaGuildFrameInfoTabFrame:Hide();  -- Hide all other pages (in this case only one).
		
	self:Print("v" .. version .. " loaded")
end


function MG:JoinMetaGuildChannels()
	--This will allow us to join the private guild channel.
	--It will use the password that came with the 
	valid = 1
	
	--self:Print("Guild Channel Value: "..GetChannelName(self.db.realm.guildChannel))
	
	if self.db.realm.guildChannel ~= nil then
		if(GetChannelName(self.db.realm.guildChannel) <= 0) then
			JoinChannelByName(self.db.realm.guildChannel, self.db.realm.guildPassword);
			if(GetChannelName(self.db.realm.guildChannel) <= 0) then
				valid = 0
			else
				--self:OnChannelJoined()
			end
		end
	end

	if valid then
		self:CancelAllTimers();
	end
end

function MG:GetZone()
	--self:Print(GetZoneText())
	zone = GetRealZoneText()
	if (zone == nil) then
		zone = GetZoneText()
	end
	return zone
end

function MG:OnChannelJoined()
	--self:CancelTimer(self.joinTimer, true);
	self:CancelAllTimers();
	--self:SetSlashCmd();
	
	-- Chat magic
	addonNameUpper = self.db.realm.baseChannel;
	ChatTypeInfo[addonNameUpper] = ChatTypeInfo["CHANNEL"..GetChannelName(self.db.realm.guildChannel)];
    	ChatTypeInfo[addonNameUpper].sticky = 1;
	_G["CHAT_MSG_" .. addonNameUpper] = "metaguild";
	_G["CHAT_" .. addonNameUpper .. "_GET"] = "[MetaGuild] %s:\32";
	_G["CHAT_" .. addonNameUpper .. "_SEND"] = "MetaGuild:\32";
	
	ListChannelByName(self.db.realm.guildChannel);
	
	local frames = self:FindChatFrames();
	if not next(frames) then
		-- The channel should be assigned to at last one frame to make sense.
		MG:Print("MetaGuild is not assigned to a chat frame. Assigning to default chat frame.");

		if self.db.realm.guildChannel then
			ChatFrame_AddChannel(DEFAULT_CHAT_FRAME, self.db.realm.guildChannel);
		end
	end
	
	ChangeChatColor("CHANNEL" .. GetChannelName(self.db.realm.guildChannel), self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b); 

	-- TODO: Sometimes these messages aren't send.
	
	zone = MG:GetZone()
	--self:Print("Player Level: ".. UnitLevel("player"))
	--self:Print("Zone: ".. self:GetZone())
	--self:Print("Rank: ".. self.db.realm.rank)
	
	self:SendJoin()
end

function MG:OnChannelLeft()
	--self:RemoveSlashCmd();
	
	-- Chat magic
	addonNameUpper = self.db.realm.baseChannel;	
	ChatTypeInfo[addonNameUpper] = nil
	_G["CHAT_MSG_" .. addonNameUpper] = nil;
	_G["CHAT_" .. addonNameUpper .. "_GET"] = nil;
	_G["CHAT_" .. addonNameUpper .. "_SEND"] = nil;
	
	DEFAULT_CHAT_FRAME.editBox:SetAttribute("stickyType", "SAY");
	DEFAULT_CHAT_FRAME.editBox:SetAttribute("chatType", "SAY");
	--_G["ChatFrame" .. self.db.profile.chatFrame].editBox:SetAttribute("stickyType", "SAY");
	--_G["ChatFrame" .. self.db.profile.chatFrame].editBox:SetAttribute("chatType", "SAY");
	
	hash_SlashCmdList = {}
	hash_ChatTypeInfoList = {}
end


function MG:OnDisable()
	-- Unhook, Unregister Events, Hide frames that you created.
	-- You would probably only use an OnDisable if you want to 
	-- build a "standby" mode, or be able to toggle modules on/off.
	
	if(self.db.profile.enabled == true) then
		return
	end
	
	self:UnregisterComm(string.upper(self.db.realm.baseChannel))
	
	if self.db.realm.GuildChannel then 
		LeaveChannelByName(self.db.realm.guildChannel);
	end
	
	self:Unhook("SendChatMessage", true);
	self:Unhook("ChatEdit_UpdateHeader", true);
	
	self.db.realm.onlinePlayers = {}
	numOnline = 0;
	self:UpdateOnlineCount();
end

function MG:CanSend()
	if (self.db.profile.enabled) then
		if(self.db.realm.guildName ~= nil and self.db.realm.guildChannel ~= nil and self.db.realm.guildPassword ~= nil) then
			if (GetChannelName(self.db.realm.guildChannel) > 0) then
				return true;
			end
		else
			self:Print("Could not send. Guild Name, Channel & Password are not set correctly")
		end
	else
		self:Print("Could not send. Addon Disabled")
	end
	return false;
end

-- EVENTS --

function MG:CHAT_MSG_CHANNEL_NOTICE(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	-- Join/Leave channel
	if(arg1 == "YOU_JOINED" and arg9 == self.db.realm.guildChannel) then
		if(self.db.profile.enabled) then
			self:OnChannelJoined();
		end
	elseif(arg1 == "YOU_LEFT" and arg9 == self.db.realm.guildChannel) then
		--if(self.db.profile.enabled) then
			self:OnChannelLeft();
			self.db.profile.enabled = false;
			self:OnDisable();
		--end
	elseif(self.db.profile.enabled) then
		self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 2);
	end
end

function MG:CHAT_MSG_CHANNEL_JOIN(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	if(arg9 == self.db.realm.guildChannel) then
		local name = arg2;
		--ListChannelByName(self.db.realm.guildChannel);
		if(self.db.realm.onlinePlayers[name] == nil) then
			--self.db.realm.onlinePlayers[name] = { ["main"] = "", ["status"] = 4}
		end
		self:Print("Updating MetaGuild from Channel Join")
		self:SendUpdate();
		self:UpdateOnlineCount();
	end
end

function MG:CHAT_MSG_CHANNEL_LEAVE(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	if(arg9 == self.db.realm.guildChannel and arg2 ~= UnitName("player")) then
		local name = arg2;
		--self:SystemMessage(player .. " left");
		self.db.realm.onlinePlayers[name] = nil;
		self:UpdateOnlineCount()
		
		local infoFrame = getglobal("MetaGuild_PlayerInfoFrame")
		local RosterFrame = getglobal("MetaGuildRosterFrame")
		if infoFrame:IsShown() and name == infoFrame.name then
			infoFrame:Hide();
			RosterFrame.selectedGuildMember = nil
		end
		
	end
end

function MG:CHAT_MSG_CHANNEL(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	if(arg9 == MG.db.realm.guildChannel) then
		if(string.sub(arg1, 0, 1) == specialChar) then
			text = self:parseMessage(arg2, string.sub(arg1, 1))
			
			if(text) then
				self:SystemMessage(text)
			end
		end
	end
end

function MG:CHAT_MSG_SYSTEM(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	-- AFK/DND status changed
	msg = arg1;
	local initStatus = status
	if string.find(msg, L["AfkMessage"]) then
		status = 2;
	elseif string.find(msg, L["RemoveAfkMessage"]) then
		status = 1;
	elseif string.find(msg, L["DndMessage"]) then
		status = 3;
	elseif string.find(msg, L["RemoveDndMessage"]) then
		status = 1;
	end
	--self:Print("Printing MetaGuild from Chat Message: "..msg)
	if status ~= initStatus then
		self:SendUpdate();
	end
end

function MG:ZONE_CHANGED_NEW_AREA()
	-- Zone changed
	self:SendUpdate();
end

function MG:ACHIEVEMENT_EARNED(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	local id = arg1;
	
	local link = GetAchievementLink(id);
	
	link = self:removeLinks(link)
	self:Print("About to send Link: "..link)
	SendChatMessage(specialChar .. "ACHIEVEMENT" .. specialChar .. link .. specialChar, self.db.realm.guildChannel);
end

function MG:TRADE_SKILL_SHOW(eventName, ...)
	if not TradeSkillFrameTitleText then 
			return 
	end
	
	if (IsTradeSkillLinked()) then
		return
	end
	
	if (GetTradeSkillListLink() == nil) then
		return
	end
	
	local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine();
	if (self.db.realm.professions[tradeskillName] == nil or self.db.realm.professions[tradeskillName].skill <= currentLevel) then
		self.db.realm.professions[tradeskillName] = {};
		self.db.realm.professions[tradeskillName].link = GetTradeSkillListLink();
		self.db.realm.professions[tradeskillName].skill = currentLevel;
	end
	if (self.db.char.professions[tradeskillName] == nil) or (self.db.char.professions[tradeskillName].skill <= currentLevel) then
		self.db.char.professions[tradeskillName] = {};
		self.db.char.professions[tradeskillName].link = GetTradeSkillListLink();
		self.db.char.professions[tradeskillName].skill = currentLevel;
	end
end

function MG:PLAYER_LEVEL_UP(...)
	MG:SendUpdate();
end

function MG:CHAT_MSG_CHANNEL_LIST(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	if(arg9 == self.db.realm.guildChannel) then
		for k,v in pairs(self:explode(", ", arg1)) do
			if(string.sub(v, 0, 1) == "*") then
				v = string.sub(v, 2);
			end
			if(self.db.realm.onlinePlayers[v] == nil) then
				--self.db.realm.onlinePlayers[v] = { ["main"] = "", ["status"] = 4}
			end
		end
		self:UpdateOnlineCount();
	end
end

-- HOOKS --
function MG:SendChatMessage(msg, chan, language, target)
	--self:Print("Chan: "..chan)
	local channelId = GetChannelName(self.db.realm.guildChannel)
	if((channelId ~= 0) and (chan == self.db.realm.guildChannel)) then
		chan = "CHANNEL";
		target = channelId;
		--if(string.sub(msg, 0, 1) ~= specialChar) then
			msg = self:removeLinks(msg);
		--end
	end
	--print("Sending Chat Message: "..msg)
	self.hooks["SendChatMessage"](msg, chan, language, target);
end

function MG:ChatEdit_UpdateHeader(editBox)
	self.hooks["ChatEdit_UpdateHeader"](editBox);
	
	addonNameUpper = self.db.realm.baseChannel;
	if (editBox:GetAttribute("chatType") == addonNameUpper) then
		editBox:SetTextColor(self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b);
		_G[editBox:GetName().."Header"]:SetTextColor(self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b);
    end
end

function MetaGuildChatFilter(ChatFrameSelf, event, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12 = ...;
	if(arg9 == MG.db.realm.guildChannel and MG.db.profile.filterDisabled == false) then
		if(string.sub(arg1, 0, 1) == specialChar) then
			return true;
		else
			arg1 = self:addLinks(arg1);
			arg1 = self:resolveSpecial(arg1);
			return false, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12;
		end
	end
	
	return false, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12;
end

function MetaGuildChatRemover(ChatFrameSelf, event, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	--self:Print("Arg: "..arg9)
	if MG.db.profile.filterDisabled then
		--Disabled the chat filter. Addon messages should now be shown.
		return false
	else
		if(arg9 == MG.db.realm.guildChannel) then
			return true
		end
	end
end

function MG:FindChatFrames()
	local frames = {}
	for i = 1,10 do
		for index,value in pairs({GetChatWindowChannels(i)}) do
			if (index%2==1) and (value == MG.db.realm.baseChannel) then
				tinsert(frames, i);
			end
		end
	end
	return frames
end

function MG:SystemMessage(msg)
	for i,v in pairs(self:FindChatFrames()) do
		_G["ChatFrame" .. v]:AddMessage("[" .. self.db.profile.chatTag .. "] " .. msg, self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b);
	end
end

function MG:OnCommReceived(prefix, msg, distribution, sender)
	self:Print("Comm Recieved - Prefix: "..prefix)
	self:Print("Comm Recieved - BaseChannel: "..self.db.realm.baseChannel)
	self:Print("Comm Recieved - Distribution: "..distribution)
	self:Print("Comm Recieved - Sender: "..sender)
	self:Print("Comm Recieved - Message: "..msg)
	
	--self:Print("Sender: "..sender)
	if(prefix == self.db.realm.baseChannel) and (distribution == "WHISPER" or distribution == "CHANNEL") then
		--> This area is for inviting new guild members<--

		-- Invite
		--local invitestring = format(self:CreateMsgTemplate(7), "INVITE", "(%a+)", "(%a+)", "(.+)", "(.+)", "(.+)", "(.+)")
		local first, last, target, sender, sendTime, guildName, guildChannel, guildPassword = string.find(msg, self:GetMsgFormatString("INVITE"))

		if(first ~= nil) then
			
			self:Print("senderTimeCode: "..sendTime)
			self:Print("localTimeCode: "..time())
			local timeOffset = self:GetTimeCodeOffset(sendTime)
			self:Print("timeOffset: "..timeOffset)
			if(sender == nil) then
				return
			end
			if(guildName ~= nil and guildChannel ~= nil and guildPassword ~= nil) then
				self:ConfirmGuildInvite(sender, guildName, guildChannel, guildPassword, timeOffset)
			end
			return
		end


		-- Kick
		--local kickstring = format(self:CreateMsgTemplate(7), "KICK", "(%a+)", "(%a+)", "(.+)", "(.+)", "(.+)", "(.+)")
		--self:Print("KickString: "..kickstring)
		--first, last, target, sender, sendTime, guildName, guildChannel, guildPassword = string.find(msg, self:GetMsgFormatString("KICK"))
		local first, last, target, sender, sendTime, guildName, guildChannel, guildPassword = string.find(msg, self:GetMsgFormatString("KICK"))

		if(first ~= nil) then
			if(sender == nil) then
				print("No sender from kick. Will not kick.")
				return
			else
				self:Print("Recived a kick request from "..sender)
			end

			if(guildName == self.db.realm.guildName and guildChannel == self.db.realm.guildChannel and guildPassword == self.db.realm.guildPassword) then
				self:Print("About to Kick")
				self:SendRemove(sender)
				print("Would have kicked "..target.." at this point")
			else
				self:Print("GuildName: Sent-"..guildName..", Stored-"..self.db.realm.guildName)
				self:Print("GuildChannel: Sent-"..guildChannel..", Stored-"..self.db.realm.guildChannel)
				self:Print("GuildPassword: Sent-"..guildPassword..", Stored-"..self.db.realm.guildPassword)			
			end
			return
		end
		
		
		-- Promote
		--local promotestring = format(self:CreateMsgTemplate(5), "PROMOTE", "(%a+)", "(%a+)", "(.+)", "(.+)")
		--self:Print("PromoteString: "..promotestring)
		local first, last, target, sender, sendTime, guildName = string.find(msg, self:GetMsgFormatString("PROMOTE"))
		if(first ~= nil) then
			if(sender == nil) then
				return
			end
			if(guildName == self.db.realm.guildName) then
				self:DoPromote()
			end
			return
		end
		
		-- Demote
		--local demotestring = format(self:CreateMsgTemplate(5), "DEMOTE", "(%a+)", "(%a+)", "(.+)", "(.+)")
		--self:Print("DemoteString: "..demotestring)
		local first, last, target, sender, sendTime, guildName = string.find(msg, self:GetMsgFormatString("DEMOTE"))
		if(first ~= nil) then
			if(sender == nil) then
				return
			end
			if(guildName == self.db.realm.guildName) then
				self:DoDemote()
			end
			return
		end
	end

	
	if(message == "REQ") then
		local tradelist = "";
		if (self.db.realm.tradeskillPerChar == true) then
			for k,v in pairs(self.db.char.professions) do
				if(v.link ~= nil) then
					tradelist = tradelist .. v.link .. specialChar;
				end
			end
		else
			for k,v in pairs(self.db.realm.professions) do
				if(v.link ~= nil) then
					tradelist = tradelist .. v.link .. specialChar;
				end
			end
		end
		self:SendCommMessage(self.db.realm.baseChannel, tradelist, "WHISPER", sender, "NORMAL");
	else
		local playerFrame = _G["MetaGuild_PlayerInfoFrame"];
		if (playerFrame:IsShown()) and (playerFrame.name == sender) then
			local tradeskilltext = "<html><body><p>";
			for k,v in ipairs(self:explode(specialChar, message)) do
				tradeskilltext = tradeskilltext .. v .. "<br/>";
				--_G["MetaGuild_PlayerInfoFrame_Tradeskills"]:AddMessage(v);
			end
			tradeskilltext = tradeskilltext .. "</p></body></html>";
			_G["MetaGuild_PlayerInfoFrame_Tradeskills"]:SetText(tradeskilltext);
		end
	end
end


function MG:GetMsgFormatString(formatType)
	local result = "BOGUS"
	if formatType == "JOIN" then
		--main, class, classLocal, level, zone, rank
		result = format(self:CreateMsgTemplate(7), "JOIN", "(.+)", "(%a+)", "(.+)", "(%d%d?)", "(.+)", "(.*)")
	end
	
	if formatType == "UPDATE" then
		--main, class, level, zone, status, rank
		result = format(self:CreateMsgTemplate(7), "UPDATE", "(%a+)", "(%a+)", "(%d%d)", "(.+)", "(%d)", "(%d)")
	end
	
	if formatType == "INVITE" then
		--target, sender, sendTime, guildName, guildChannel, guildPassword
		result = format(self:CreateMsgTemplate(7), "INVITE", "(%a+)", "(%a+)", "(%d+)", "(.+)", "(.+)", "(.+)")
	end
	
	if formatType == "REMOVE" then
		--local removestring = format(self:CreateMsgTemplate(7), "REMOVE", self.db.realm.main, kicker, self.db.realm.guildName, self.db.realm.guildChannel, self.db.realm.guildPassword, time());	
		--main, kicker, guildName, guildChannel, guildPassword, sendTime
		result = format(self:CreateMsgTemplate(7), "REMOVE", "(%a+)", "(%a+)", "(.+)", "(.+)", "(.+)",  "(%d+)")
	end
	
	if formatType == "KICK" then
		----target, sender, sendTime, guildName, guildChannel, guildPassword
		result = format(self:CreateMsgTemplate(7), "KICK", "(%a+)", "(%a+)", "(%d+)", "(.+)", "(.+)", "(.+)")
	end	

	if formatType == "PROMOTE" then
		--target, sender, sendTime, guildName
		result = format(self:CreateMsgTemplate(5), "PROMOTE", "(%a+)", "(%a+)", "(%d+)", "(.+)")
	end

	if formatType == "DEMOTE" then
		--target, sender, sendTime, guildName
		result = format(self:CreateMsgTemplate(5), "DEMOTE", "(%a+)", "(%a+)", "(%d+)", "(.+)")
	end
	
	if formatType == "MOTD" then
		--motd, nr, author
		result = format(self:CreateMsgTemplate(4), "MOTD", "(.+)", "(%d+)", "(%a+)")
	end
	
	if formatType == "TIMECODE" then
		--main, rank, sendTime
		result = format(self:CreateMsgTemplate(4), "TIMECODE", "(%a+)", "(%d)", "(%d+)")
	end
	
	return result
end

function MG:parseMessage(player, msg)
	local ret = nil;

	-- TimeCodeUpdate
	local first, last, main, rank, sendTime = string.find(msg, self:GetMsgFormatString("TIMECODE"))	
	if(first ~= nil) then
		local timeOffset = self:GetTimeCodeOffset(sendTime)
		if rank == 3 then
			--This is the guild master and use this new value for your timecode Offset
			self.db.realm.timeOffset = timeOffset
		end
		self:Print("TimeCodeUpdate by: "..main.." ("..self:GetRankName(tonumber(rank))..")")
		self:Print("SendTime: "..sendTime)
		self:Print("TimeOffset: "..timeOffset)
		return
	end
	
	-- A player updates  (rough max 70 chars)
	local first, last, main, class, level, zone, status, rank = string.find(msg, self:GetMsgFormatString("UPDATE"));	
	if(first ~= nil) then
		self.db.realm.onlinePlayers[player] = { ["main"] = main, ["status"] = status }
		self:UpdateOfflinePlayerlist(player, {["main"] = main, ["class"] = class, ["level"] = tonumber(level), ["zone"] = zone, ["time"] = self:GetTimeCode(), ["rank"] = tonumber(rank)})
		return
	end

	
	
	-- A player joins
	local first, last, main, class, classLocal, level, zone, rank = string.find(msg, self:GetMsgFormatString("JOIN"));
	if(first ~= nil) then
		if(player ~= main) then
			self:SystemMessage(player .. "(" .. main .. ") joined.");
			ret = player .. "(" .. main .. ") joined.";
		else
			self:SystemMessage(player .. " joined.");
			ret = player .. " joined.";
		end
		--self.db.realm.onlinePlayers[player] = { ["name"] = player, ["main"] = main, ["class"] = class, ["classLocal"] = classLocal, ["level"] = level, ["zone"] = zone, ["status"] = 1, ["rank"] = rank }
		self.db.realm.onlinePlayers[player] = { ["main"] = main, ["status"] = 1 }
		--self:UpdateOfflinePlayerlist(self.db.realm.onlinePlayers[player])  Note: This should be happening in the UpdateOnlineCount() function
		self:UpdateOfflinePlayerlist(player, {["main"] = main, ["class"] = class, ["level"] = level, ["zone"] = zone, ["time"] = self:GetTimeCode(), ["rank"] = tonumber(rank)})
		self:SendUpdate();
		--self:SendOldMotd();
		return ret;
	end
	
	-- A player leaves
	local first, last = string.find(msg, "LEAVE"..specialChar);
	if(first ~= nil) then
		
		self:SystemMessage(player .. " left");
		self.db.realm.onlinePlayers[player] = nil;
		self:UpdateOnlineCount();
		self:Print("Player Leaves... Message parse")
		return; --player .. " left";
	end	
	
	-- A player is being Removed  (rough max 70 chars)
	local first, last, main, kicker, guildName, guildChannel, guildPassword, time = string.find(msg, self:GetMsgFormatString("REMOVE"));	
	if(first ~= nil) then
		if self.db.realm.offlinePlayers[main] ~= nil and guildName == self.db.realm.guildName and guildChannel == self.db.realm.guildChannel and guildPassword == self.db.realm.guildPassword then
			self:DoRemove(main, kicker, time)
		end
		return
	end
	
	
	
	-- Motd
	local first, last, motd, nr, author = string.find(msg, self:GetMsgFormatString("MOTD"));	
	if(first ~= nil) then
		if(author == nil) then
			return
		end
		if(self.db.realm.motd.nr == nil or self.db.realm.motd.nr < nr) then
			self.db.realm.motd = { ["msg"] = motd, ["nr"] = nr, ["author"] = author }
			if(player == author) then
				--self:SystemMessage(format(L["NewMotd"], player, motd));
				ret = format(L["NewMotd"], player, motd);
			end
		end
		
		self:UpdateMotd();
		return ret;
	end
	
	-- Achievement
	first, last, link = string.find(msg, specialChar.."ACHIEVEMENT" .. specialChar .. "(.+)" .. specialChar);
	if(first ~= nil) then
		self:Print("Just had an Achievement show up")
		if(self.db.profile.showAchiev) then
			self:Print(format(L["AchievementEarned"], player, self:addLinks(link)));
			SendChatMessage(format(L["AchievementEarned"], player, self:addLinks(link)), self.db.realm.guildChannel);
			return format(L["AchievementEarned"], player, self:addLinks(link));
		end
	end
end

function MG:GetCharacterInfo(name)	
	local main = self:GetMainFromName(name)

	if self.db.realm.offlinePlayers[main] ~= nil and self.db.realm.offlinePlayers[main].characters ~= nil then
		local character = self.db.realm.offlinePlayers[main].characters[name]
		character.name = name
		character.main = main
		if self.db.realm.onlinePlayers[name] ~= nil then
			character.status = tonumber(self.db.realm.onlinePlayers[name].status)
		else
			character.status = "Offline"
		end
		character.rank = tonumber(self.db.realm.offlinePlayers[main].rank)
		
		character.bgrating = 2200
		character.arenarating = 1234
		
		return character
	else
		--self:Print("No character named "..name.." could be found in the database.")
		return nil
	end
end

function MG:CharacterIsOnline(characterName)
	for k, v in pairs(self.db.realm.onlinePlayers) do
		if characterName == k then
			return true
		end
	end
	return false
end


function MG:UpdateOfflinePlayerlist(name, data)
	local main = nil
	if data["main"] ~= nil then
		main = data["main"]
	end

	--We need to have a main selected at this point. Without it, we cannot sort the data.
	if main ~= nil then		
		--Lets see if the main has already been saved to the db.
		if self.db.realm.offlinePlayers[main] == nil then		
			--The main does not exist. This could be because its a new record, or that the user has switched mains.
			--Lets loop through the offline characters and see if the new main exists already. If it does, the user switched mains.
			--And we will need to update the database to reflect that.
			local oldMain = nil
			for a, b in pairs(self.db.realm.offlinePlayers) do --"a" is the main, "b" is the list of characters
				for c, d in pairs(b["characters"]) do --"c" is the character name, "d" is the attributes
					if c == main then
						oldMain = a
						--We found the main in the database already.
						--Set oldMain so it will update the database to reflect it as the main
					end
				end
			end
			
			if oldMain == nil then
				self.db.realm.offlinePlayers[main] = {["characters"] = {}, ["rank"] = 0}
			else
				self.db.realm.offlinePlayers[main] = self.db.realm.offlinePlayers[oldMain]  --assign the old record to the new main key
				self.db.realm.offlinePlayers[oldMain] = nil --Remove the old main key				
			end
		end

		if self.db.realm.offlinePlayers[main] == nil then
			self:Print("Main ("..main..") does not exist in the data base for some reason. This should not happen at this point.")
		end


		--The main either existed or has been created. 
		--Now lets see if the current character has been placed in the database.		
		if self.db.realm.offlinePlayers[main].characters[name] == nil then
			--The character does not exist yet in the character array. We will create it so the next part can add the data to it.
			self.db.realm.offlinePlayers[main].characters[name] = {["class"]="", ["level"]="", ["zone"]="", ["time"]="", ["public"]="", ["officer"]=""}
		end

		--The character has already been assigned to the database. Now all that is left is to update it.
		for k,v in pairs(data) do --loop through all the new character data. (k,v) == (attr, data)
			if k == "time" and v == nil then
				v = self:GetTimeCode()
			end
			if k == "rank" then
				self.db.realm.offlinePlayers[main].rank = v
			else
				self.db.realm.offlinePlayers[main].characters[name][k] = v
			end
		end
	end
	self:UpdateOnlineCount()
end

function MG:RepairOfflineList()
	local isValid
	for a, b in pairs(self.db.realm.offlinePlayers) do
		isValid = false
		if b.characters ~= nil then
			for c,d in pairs(b.characters) do
				if c == a then
					isValid = true
				end
			end
		end
		if isValid == false then
			self.db.realm.offlinePlayers[a] = nil
		end
	end
	self:UpdateOnlineCount()
	
end

function  MG:DisplayPlayerList(playerList, isOffline)
	for k, v in pairs(playerList) do
		if v.main == "" then
			v.main = "ERROR"
		end

		local charactersStr = ""
		if isOffline then
			charactersStr = ""
			for y,z in pairs(v.characters) do
				if charactersStr == "" then
					charactersStr = (charactersStr..y.."-"..z.level.."-"..z.class)
				else
					charactersStr = (charactersStr..", "..y.."-"..z.level.."-"..z.class)
				end
			end
			self:Print("Main: "..k..", Characters: ["..charactersStr.."]")
		else
			self:Print("Name: "..k..", Main:"..v.main..", Status: "..self:GetStatusText(v.status))
			--self:Print("Name: "..k..", Main:"..v.main)

		end
	end
end

function MG:UpdateOnlineCount()
	local i = 0
	local j = 0
	
	for k, v in pairs(self.db.realm.onlinePlayers) do
		i = i+1
	end
	
	for k, v in pairs(self.db.realm.offlinePlayers) do
		j = j+1
	end	
	numOnline = i
	dataobj.text = "MG (" .. numOnline .. ")"
	MetaGuildFrameMembersCount:SetText(format(L["Main/MembersOnlineCount"], i, j))
	MetaGuildRoster_Update()
end

function MG:UpdateMotd()
	local frame = "MetaGuild_OnlineFrame";
	if(self.db.realm.motd.msg ~= nil) and (self.db.realm.motd.author ~= nil) and (self.db.realm.motd.nr ~= nil) then
	-->	_G[frame .. "_Motd"]:SetText(format(L["Motd"], self.db.realm.motd.msg, self.db.realm.motd.author, date("%c", nr))); -- %d.%m.%y %H:%M
	end
end

--> Send Functions <--

function MG:SendInvite(playerName)
	if (self:CanSend() and tonumber(self.db.realm.rank) >= 2) then
		if strlower(playerName) ~= strlower(UnitName("player")) then
			local invitestring = format(self:CreateMsgTemplate(7), "INVITE", playerName, UnitName("player"), self:GetTimeCode(), self.db.realm.guildName, self.db.realm.guildChannel, self.db.realm.guildPassword);
			self:Print("InviteString: "..invitestring)
			self:SendCommMessage(string.upper(self.db.realm.baseChannel), invitestring, 'WHISPER', playerName)
		else
			self:Print("Cannot re-invite yourself to the guild.")
		end
	end
end

function MG:SendKick(playerName)
	if (self:CanSend()) then
		--if strlower(playerName) ~= strlower(UnitName("player")) then
			--local kickstring = format(self:CreateMsgTemplate(7), "KICK", playerName, UnitName("player"), time(), self.db.realm.guildName, self.db.realm.guildChannel, self.db.realm.guildPassword)
			local kickstring = format(self:CreateMsgTemplate(7), "KICK", playerName, UnitName("player"), self:GetTimeCode(), self.db.realm.guildName, self.db.realm.guildChannel, self.db.realm.guildPassword)
			--self:Print("KICK-kickString: "..kickstring)
			self:SendCommMessage(self.db.realm.baseChannel, kickstring, 'WHISPER', playerName)
		--else
		--	self:Print("Cannot kick yourself from the guild.")
		--end
	end	
end


function MG:SendPromote(playerName)
	if (self:CanSend()) then
		local promotestring = format(self:CreateMsgTemplate(5), "PROMOTE", playerName, UnitName("player"), time(), self.db.realm.guildName);
		self:Print("Promote-PromoteString: "..promotestring)
		self:SendCommMessage(self.db.realm.baseChannel, promotestring, 'WHISPER', playerName)
	end
end

function MG:SendDemote(playerName)
	if (self:CanSend()) then
		local demotestring = format(self:CreateMsgTemplate(5), "DEMOTE", playerName, UnitName("player"), time(), self.db.realm.guildName);
		self:Print("Demote-DemoteString: "..demotestring)
		self:SendCommMessage(self.db.realm.baseChannel, demotestring, 'WHISPER', playerName)
	end
end

function MG:SendUpdate()
	if (self:CanSend()) then
		local updatestring = format(self:CreateMsgTemplate(7), "UPDATE", self.db.realm.main, select(2,UnitClass("player")), UnitLevel("player"), self:GetZone(), status, self.db.realm.rank);
		SendChatMessage(updatestring, self.db.realm.guildChannel);
	end
end

function MG:SendTimeCode()
	if (self:CanSend()) then
		local timecodestring = format(self:CreateMsgTemplate(4), "TIMECODE", self.db.realm.main, self.db.realm.rank, self:GetTimeCode());
		SendChatMessage(timecodestring, self.db.realm.guildChannel);
	end

end

function MetaGuildSendJoin()
	self:SendJoin()
end

function MG:SendJoin()
	if (self:CanSend()) then
		--self:Print("BOABABDS")
		local joinstring = format(self:CreateMsgTemplate(7), "JOIN", self.db.realm.main, select(2,UnitClass("player")), UnitClass("player"), UnitLevel("player"), self:GetZone(), self.db.realm.rank);	
		SendChatMessage(joinstring, string.upper(self.db.realm.guildChannel));
	else
		self:Print("Could not Send")
	end
end

function MG:SendRemove(kicker)
	if (self:CanSend() and kicker ~= nil) then
		local removestring = format(self:CreateMsgTemplate(7), "REMOVE", self.db.realm.main, kicker, self.db.realm.guildName, self.db.realm.guildChannel, self.db.realm.guildPassword, time());	
		SendChatMessage(removestring, string.upper(self.db.realm.guildChannel));
	else
		self:Print("Could not Send")
	end
end


function MG:SendNewMotd(motd)
	if (self:CanSend()) then
		local motdstring = format(self:CreateMsgTemplate(4), "MOTD", motd, time(), UnitName("player"));
		SendChatMessage(motdstring, self.db.realm.guildChannel);
	end
end

function MG:SendOldMotd()
	if(self.db.realm.motd.msg ~= nil) then
		local motdstring = format(self:CreateMsgTemplate(4), "MOTD", self.db.realm.motd.msg, self.db.realm.motd.nr, self.db.realm.motd.author);
		SendChatMessage(motdstring, self.db.realm.guildChannel);
	end
end

function MG:resolveSpecial(msg)
	msg = string.gsub(msg, format(colorChar, 'r'), red);
	msg = string.gsub(msg, format(colorChar, 'g'), green);
	msg = string.gsub(msg, format(colorChar, 'b'), blue);
	msg = string.gsub(msg, format(colorChar, 'n'), "\124r");
	
	return msg;
end

function MG:removeLinks(msg)
	msg = string.gsub(msg, "|c", specialLinkChar.."c");
	msg = string.gsub(msg, "|Hitem:", specialLinkChar.."Hitem:");
    msg = string.gsub(msg, "|Henchant:", specialLinkChar.."Henchant:");
	msg = string.gsub(msg, "|Hspell:", specialLinkChar.."Hspell:");
	msg = string.gsub(msg, "|Hquest:", specialLinkChar.."Hquest:");
	msg = string.gsub(msg, "|Htrade", specialLinkChar.."Htrade");
	msg = string.gsub(msg, "|Hglyph", specialLinkChar.."Hglyph");
	msg = string.gsub(msg, "|Htalent", specialLinkChar.."Htalent");
	msg = string.gsub(msg, "|Hachievement", specialLinkChar.."Hachievement");
	msg = string.gsub(msg, "|Hinstancelock", specialLinkChar.."Hinstancelock");
    msg = string.gsub(msg, "|h|r", specialLinkChar.."h"..specialLinkChar.."r");
    msg = string.gsub(msg, "|h", specialLinkChar.."h");
	msg = string.gsub(msg, "|r", specialLinkChar.."r");
	msg = string.gsub(msg, "|T", specialLinkChar.."T");
	msg = string.gsub(msg, "|t", specialLinkChar.."t");
	
	return msg;
end

function MG:addLinks(msg)
	msg = string.gsub(msg, specialLinkChar.."c", "|c");
	msg = string.gsub(msg, specialLinkChar.."Hitem:", "|Hitem:");
    msg = string.gsub(msg, specialLinkChar.."Henchant:", "|Henchant:");
	msg = string.gsub(msg, specialLinkChar.."Hspell:", "|Hspell:");
	msg = string.gsub(msg, specialLinkChar.."Hquest:", "|Hquest:");
	msg = string.gsub(msg, specialLinkChar.."Htrade:", "|Htrade:");
	msg = string.gsub(msg, specialLinkChar.."Hglyph:", "|Hglyph:");
	msg = string.gsub(msg, specialLinkChar.."Htalent:", "|Htalent:");
	msg = string.gsub(msg, specialLinkChar.."Hachievement:", "|Hachievement:");
	msg = string.gsub(msg, specialLinkChar.."Hinstancelock:", "|Hinstancelock:");
    msg = string.gsub(msg, specialLinkChar.."h"..specialLinkChar.."r", "|h|r");
    msg = string.gsub(msg, specialLinkChar.."h", "|h");
	msg = string.gsub(msg, specialLinkChar.."r", "|r");
	msg = string.gsub(msg, specialLinkChar.."T", "|T");
	msg = string.gsub(msg, specialLinkChar.."t", "|t");
	
	return msg;
end

function MetaGuild_Toggle()
	if (MetaGuildFrame:IsShown()) then
		MetaGuildFrame:Hide();
	else
		self:InitLocalization();
		MetaGuildFrame:Show();
	end 
end


function MetaGuildFrame_TabClicked(self)
	local tabIndex = self:GetID();
	if (not self:GetParent().numTabs) then
		self:GetParent().numTabs = 4
	end
	
	PanelTemplates_SetTab(MetaGuildFrame, tabIndex);
	--print(self:GetText().." ("..tabIndex..")")
	--print(self:GetParent().numTabs)
	if (tabIndex == 1) then
		MetaGuildFrameGuildTabFrame:Show()
		MetaGuildFrameRosterTabFrame:Hide()
		MetaGuildFrameNewsTabFrame:Hide()
		MetaGuildFrameInfoTabFrame:Hide()
		MetaGuild_PlayerinfoShow("", true)
	elseif (tabIndex == 2) then
		MetaGuildFrameGuildTabFrame:Hide()
		MetaGuildFrameRosterTabFrame:Show()
		MetaGuildFrameNewsTabFrame:Hide()
		MetaGuildFrameInfoTabFrame:Hide()
	elseif (tabIndex == 3) then
		MetaGuildFrameGuildTabFrame:Hide()
		MetaGuildFrameRosterTabFrame:Hide()
		MetaGuildFrameNewsTabFrame:Show()
		MetaGuildFrameInfoTabFrame:Hide()
		MetaGuild_PlayerinfoShow("", true)
	elseif (tabIndex == 4) then
		MetaGuildFrameGuildTabFrame:Hide()
		MetaGuildFrameRosterTabFrame:Hide()
		MetaGuildFrameNewsTabFrame:Hide()
		MetaGuildFrameInfoTabFrame:Show()
		MetaGuild_PlayerinfoShow("", true)
	end

	MG:UpdateOnlineCount()
end


function MG:GetTradeskills(name)
	self:SendCommMessage("MetaGuild", "REQ", "WHISPER", name, "NORMAL");
end

function MG:explode(div,str)
  if (div=='') then return false end
  local pos,arr = 0,{}
  -- for each divider found
  for st,sp in function() return string.find(str,div,pos,true) end do
    table.insert(arr,string.sub(str,pos,st-1)) -- Attach chars left of current divider
    pos = sp + 1 -- Jump past current divider
  end
  table.insert(arr,string.sub(str,pos)) -- Attach chars right of last divider
  return arr
end

-- Profile changes
function MG:LoadedNewProfile()
	if(self.db.profile.enabled) then
		self:OnEnable();
		self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 2);
	end
end

function MG:OnProfileReset()
	self:OnDisable();
end

function MG:LeaveProfile()
	if(self.db.profile.enabled) then
		-- Disable, but set to enabled without enabling
		self.db.profile.enabled = false;
		self:OnDisable();
		self.db.profile.enabled = true;
	end
end

function MetaGuildInvite(playerName)
	if self.db.realm.rank >= 2 then
		--self:Print('Invited '.. playerName)
		self:SendInvite(playerName)
	end
end

function MetaGuildKick(playerName)
	--self:Print('About to Kick '.. playerName)
	local main = self:GetMainFromName(playerName)	
	if main ~= nil and self.db.realm.offlinePlayers[main] ~= nil then
		--if tonumber(self.db.realm.rank) > tonumber(self.db.realm.offlinePlayers[main].rank) then
			--self:Print('Kicked '.. playerName)
			self:SendKick(playerName)
		--else
		--	self:Print('You do not have the rank to kick '.. playerName)
		--end
	end
end

function MG:InitPopupDialogs()
	StaticPopupDialogs["REMOVE_METAGUILDMEMBER"] = {
		text = format(REMOVE_GUILDMEMBER_LABEL, "XXX"),
		button1 = YES,
		button2 = NO,
		OnAccept = function(self)
			local name, main = GetMetaGuildRosterSelection()
			MetaGuildKick(name)
			getglobal("MetaGuild_PlayerInfoFrame"):Hide()
		end,
		OnShow = function(self)
			local name, main = GetMetaGuildRosterSelection()
			self.text:SetFormattedText(L["PlayerInfoFrame/RemoveWarning"], main);
			--format(L["Main/MembersOnlineCount"], i, j)
		end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1
	};
	StaticPopupDialogs["ADD_METAGUILDMEMBER"] = {
		text = L["Info/Add_Member_Title"],
		button1 = L["Accept"],
		button2 = L["Cancel"],
		hasEditBox = 1,
		--autoCompleteParams = AUTOCOMPLETE_LIST.GUILD_INVITE,
		maxLetters = 12,
		OnAccept = function(self)
			MetaGuildInvite(self.editBox:GetText());
			
		end,
		OnShow = function(self)
			self.editBox:SetFocus();
		end,
		OnHide = function(self)
			ChatEdit_FocusActiveWindow();
			self.editBox:SetText("");
		end,
		EditBoxOnEnterPressed = function(self)
			local parent = self:GetParent();
			MetaGuildInvite(parent.editBox:GetText());
			parent:Hide();
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide();
		end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1
	};
	StaticPopupDialogs["SET_METAGUILDPLAYERNOTE"] = {
		
		text = SET_GUILDPLAYERNOTE_LABEL,
		button1 = L["Accept"],
		button2 = L["Cancel"],
		hasEditBox = 1,
		maxLetters = 31,
		editBoxWidth = 260,
		OnAccept = function(self)
			local name, main = GetMetaGuildRosterSelection()
			MetaGuildRosterSetPublicNote(name,main, self.editBox:GetText())
			MetaGuild_PlayerinfoUpdate(name, main)
		end,
		OnShow = function(self)
			local name, main = GetMetaGuildRosterSelection()
			if name ~= nil and main ~= nil then
				local character = MetaGuildGetCharacterInfo(name)
				if character.public ~= nil then
					self.editBox:SetText(character.public)
				end
				self.editBox:SetFocus()
			else
				Print("rosterSelection is Nil for some reason... This should not really happen.")
			end
		end,
		OnHide = function(self)
			ChatEdit_FocusActiveWindow();
			self.editBox:SetText("");
		end,
		EditBoxOnEnterPressed = function(self)
			local parent = self:GetParent()
			local name, main = GetMetaGuildRosterSelection()
			MetaGuildRosterSetPublicNote(name, main, parent.editBox:GetText())
			MetaGuild_PlayerinfoUpdate(name, main)
			parent:Hide()
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide();
		end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1
	};
	StaticPopupDialogs["SET_METAGUILDOFFICERNOTE"] = {
		text = SET_GUILDOFFICERNOTE_LABEL,
		button1 = L["Accept"],
		button2 = L["Cancel"],
		hasEditBox = 1,
		maxLetters = 31,
		editBoxWidth = 260,
		OnAccept = function(self)
			local name, main = GetMetaGuildRosterSelection()
			MetaGuildRosterSetOfficerNote(name,main, self.editBox:GetText())
			MetaGuild_PlayerinfoUpdate(name, main)
		end,
		OnShow = function(self)
			local name, main = GetMetaGuildRosterSelection()
			if name ~= nil and main ~= nil then
				local character = MetaGuildGetCharacterInfo(name)
				if character.officer ~= nil then
					self.editBox:SetText(character.officer)
				end
				self.editBox:SetFocus()
			else
				Print("rosterSelection is Nil for some reason... This should not really happen.")
			end
		end,
		OnHide = function(self)
			ChatEdit_FocusActiveWindow();
			self.editBox:SetText("");
		end,
		EditBoxOnEnterPressed = function(self)
			local parent = self:GetParent();
			GuildRosterSetOfficerNote(GetGuildRosterSelection(), parent.editBox:GetText());
			parent:Hide();
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide();
		end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1
	};
end

function MG:IsPlayersCharacter(name)
	local main = self:GetMainFromName(UnitName("player"))
	if main ~= nil and self.db.realm.offlinePlayers[main] ~= nil then
		for k, v in pairs(self.db.realm.offlinePlayers[main].characters) do
			if k == name then
				return true
			end
		end
	end
	return false
end


function MG:RemoveMainFromOnlineList(main)
	for k, v in pairs(self.db.realm.onlinePlayers) do
		if v.main == main then
			self.db.realm.onlinePlayers[k] = nil
			return
		end
	end
end

function MG:firstToUpper(str)
    return (str:gsub("^%l", string.upper))
end


function MG:GetMainFromName(name)
	local main = ""
	name = self:firstToUpper(name)
	if self.db.realm.onlinePlayers[name] ~= nil then
		--The player is online, this should be easy.
		main = self.db.realm.onlinePlayers[name].main
	else
		--The player is offline, this will be a bit more work.
		for a, b in pairs(self.db.realm.offlinePlayers) do
			for c, d in pairs(b.characters) do
				if c == name then
					main = a
				end
			end
		end
	end
	return main
end

function MG:GetRank(name)
	local main = self:GetMainFromName(name)
	if main ~= nil then 
		return tonumber(self.db.realm.offlinePlayers[main].rank)
	else
		return -1
	end
end

function MetaGuildGetCharacterInfo(name)
	return self:GetCharacterInfo(name)
end

function MetaGuildRosterSetPublicNote(name, main, t)
	if main ~= nil and name ~= nil then
		self.db.realm.offlinePlayers[main].characters[name].public = t
	else
		self:Print("Main or Name was not valid")
	end
end

function MetaGuildRosterSetOfficerNote(name, main, t)
	if main ~= nil and name ~= nil then
		self.db.realm.offlinePlayers[main].characters[name].officer = t
	else
		self:Print("Main or Name was not valid")
	end
end

function GetMetaGuildRosterSelection()
	local RosterFrame = getglobal("MetaGuildRosterFrame")
	return RosterFrame.selectedGuildMember.name, RosterFrame.selectedGuildMember.main
end

function MG:ConfirmGuildInvite(sender, guildName, guildChannel, guildPassword, timeOffset)

	StaticPopupDialogs["ACCEPT_METAGUILD_INVITE"] = {
		text =  L["Info/AcceptGuildInviteText"],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			self.db.realm.timeOffset = timeOffset
			--self:SetTimeCodeOffset(timeOffset)
			self.db.realm.guildName = guildName
			self.db.realm.guildChannel = guildChannel
			self.db.realm.guildPassword = guildPassword
			self.db.realm.offlinePlayers = {}
			self.db.realm.onlinePlayers = {}
			self.db.realm.main = UnitName("player")
			self:JoinMetaGuildChannels()
			
			self:Print("Setting Main to "..self.db.realm.main)
			if self.db.realm.GuildChannel then 
				self:RegisterComm(string.upper(self.db.realm.GuildChannel))
			end
			self:ScheduleTimer(MetaGuildSendJoin, 5)

		end,
		sound = "levelup2",
		timeout = 30,
		whileDead = true,
		hideOnEscape = true,
	}

	StaticPopup_Show ("ACCEPT_METAGUILD_INVITE", sender, guildName)
end


function MG:DoRemove(main, kicker, time)
	if main ~= nil then
		if main == self.db.realm.main then
			--You are to remove yourself
			self:Print("You have been removed from "..self.db.realm.guildName)
			if self.db.realm.GuildChannel then 
				LeaveChannelByName(self.db.realm.guildChannel);
			end
			self.db.realm.guildName = ""
			self.db.realm.guildChannel = ""
			self.db.realm.guildPassword = ""
			self.db.realm.timeOffset = 0
			self.db.realm.onlinePlayers = {}
			self.db.realm.offlinePlayers = {}
		else
			self:Print(main.." was removed from "..self.db.realm.guildName)
			self:RemoveMainFromOnlineList(main)
			self.db.realm.kickedPlayers[main] = {["time"]=time, ["kicker"]=kicker, ["characters"] = self.db.realm.offlinePlayers[main].characters}
			self.db.realm.offlinePlayers[main] = nil
		end
		self:UpdateOnlineCount()
	end
end

function MG:DoPromote()
	self.db.realm.rank =  math.min(3, self.db.realm.rank + 1)
	self:SendUpdate()
end

function MG:DoDemote()
	self.db.realm.rank =  math.max(0, self.db.realm.rank - 1)
	self:SendUpdate()
end

function MG:SlashCommand(input)
	input = strlower(input)
	cmd, arg = strsplit(" ", input)
	if not input or input:trim() == "" or cmd == "usage" then
		--InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
		--LibStub("AceConfigCmd-3.0").HandleCommand(MG, "mg", "MetaGuild", input)
		MetaGuild_Toggle()
	elseif cmd == "invite" then
		if not arg or arg:trim() == "" then
		
		else
			MetaGuildInvite(arg)
		end
	elseif cmd == "kick" then
		if not arg or arg:trim() == "" then
		
		else
			MetaGuildKick(arg)
		end
	elseif cmd == "offline" then
		self:DisplayPlayerList(self.db.realm.offlinePlayers, true)
	elseif cmd == "online" then
		self:DisplayPlayerList(self.db.realm.onlinePlayers, false)
	elseif cmd == "repair" then
		self:RepairOfflineList(arg)
	elseif cmd == "time" then
		if arg == "send" then
			self:SendTimeCode()
		elseif arg == "offset" then
			self:Print("TimeOffset: "..self.db.realm.timeOffset)
		else
			self:Print("TimeCode: "..self:GetTimeCode())
		end
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(MG, "mg", "MetaGuild", input)
	end
end

function MG:tableMerge(t1, t2)
	--> Merge table2 into table1. Table 2's data will overwrite table 1 <-- 
	for k,v in pairs(t2) do
		if type(v) == "table" then
			if type(t1[k] or false) == "table" then
				tableMerge(t1[k] or {}, t2[k] or {})
			else
				t1[k] = v
			end
		else
			t1[k] = v
		end
	end
	return t1
end


--Time Functions

function MG:GetTimeCode(t)
	if t == nil then
		t = time()
	end
	t = tonumber(t)
	return (t + self.db.realm.timeOffset)
end

function MG:GetTimeCodeOffset(t)
	if t ~= nil then
		if self.db.realm.rank == 3 then
			return 0
		else
			return tonumber(t) - time()
		end
	else
		return self.db.realm.timeOffset
	end
end

function MG:TimeSinceLastTimeCode(lastTime)
	local nowDate = date("*t", self:GetTimeCode())
	local lastDate = date("*t", lastTime)
	
	--self:DisplayDateTable(nowDate)
	--self:DisplayDateTable(lastDate)
	
	return self:TimeDifference(nowDate, lastDate)
end


function MG:GetServerSeconds()
	local hour, minute = GetGameTime()
	if self.lastMinute ~= minute then
		if self.debug then
			self:Print("Updating minute to "..minute)
		end
		local _
		_,self.lastMinute = GetGameTime()
		self.secondsDifference = mod(time(), 60)
	elseif self.secondsDifference == nil then
		self.secondsDifference = 0
	end
	return format("%02d", mod(time() - self.secondsDifference, 60))
end

function MG:GetCurrentTimeCode()

	--IMPORTANT: due to the way we are getting the seconds off the server time, 
	--we must wait a minute before actually calling GetCurrentTimeCode() for best accuracy

	local serverOffset = self:GetServerOffset()
	local hour, minute = GetGameTime()
	local year, month, day, localHour, second = date("%Y"), date("%m"), date("%d"), date("%H"), date("%S")
	
	second = self:GetServerSeconds()

	if (localHour - serverOffset < 0) then
		--Re-adjust the date to refect that the server is still living out yesterday.
		--do we need to flip back a year?
		if month == 1 and day == 1 then
			year = year - 1
		end
		--do we need to flip back a month?
		if day == 1 then
			if month == 1 then
				month = 12
			else
				month = month - 1
			end
			day = self:GetDaysInMonth(month, year)
		end
	end
	
	local date = {["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["minute"] = minute, ["second"] = second}
	return self:Date2TimeCode(date)
end

function MG:PadNumber(number, padding)
	return string.format("%0"..padding.."d", number)
end

function MG:Date2TimeCode(date)
	--Must Pad the numbers to keep the format
	date.year = self:PadNumber(date.year, 4)
	date.month = self:PadNumber(date.month, 2)
	date.day = self:PadNumber(date.day, 2)
	date.hour = self:PadNumber(date.hour, 2)
	date.min = self:PadNumber(date.min, 2)
	date.sec = self:PadNumber(date.sec, 2)
	return (date.year..date.month..date.day..date.hour..date.min..date.sec)
end

function MG:TimeCode2String(timeCode)
	--self:Print("Tyep: "..type(timeCode))
	date = self:TimeCode2Date(timeCode)
	--self:Print("Date: "..self:Date2String(date))
	return date("%B %d, %Y %X",  time(date))
end






function MG:Date2String(date)
	return (date.year..", "..date.month..", "..date.day..". "..date.hour..", "..date.min..", "..date.sec)
end

function MG:TimeSinceTimeCode(oldTimeCode)
	local currentTimeCode = self:GetCurrentTimeCode()
	local currentDate = self:TimeCode2Date(currentTimeCode)
	local oldDate = self:TimeCode2Date(oldTimeCode)
	local diffDate = self:TimeDifference(currentDate, oldDate)
	--self:Print("FinalDiffDate: "..self:Date2String(diffDate))
	return diffDate
end

function MG:TimeCode2Date(timeCode)
	--self:Print("TimeCode: "..timeCode)
	local year = tonumber(string.sub(timeCode, 0, 4))
	local month = tonumber(string.sub(timeCode, 5, 6))
	local day = tonumber(string.sub(timeCode, 7, 8))
	local hour = tonumber(string.sub(timeCode, 9, 10))
	local minute = tonumber(string.sub(timeCode, 11, 12))
	local second = tonumber(string.sub(timeCode, 13, 14))
	return {["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = second}
end




function MG:GetDaysInMonth(month, year)
	local feb = 28
	if self:isLeapYear(year) then
		feb = 29
	end
	local daysInMonths = {31, feb, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
	return daysInMonths[month]
end

function MG:IsLeapYear(year)
--A year will be a leap year if it is divisible by 4 but not by 100. 
--If a year is divisible by 4 and by 100, it is not a leap year unless it is also divisible by 400. 
	if ((year % 4 == 0 and year % 100 ~= 0) or year % 400 == 0) then
		return 1
	else
		return 0
	end
end


function MG:GetLocalOffset()
	if self.localOffset ~= nil then
		return self.localOffset
	end
	local localHour = tonumber(date("%H"))
	local localMinute = tonumber(date("%M"))
	local utcHour = tonumber(date("!%H"))
	local utcMinute = tonumber(date("!%M"))
	local loc = localHour + localMinute / 60
	local utc = utcHour + utcMinute / 60
	self.localOffset = floor((loc - utc) * 2 + 0.5) / 2
	if self.localOffset >= 12 then
		self.localOffset = self.localOffset - 24
	end
	return self.localOffset
end

function MG:GetServerOffset()
	if self.serverOffset ~= nil then
		return self.serverOffset
	end
	local serverHour, serverMinute = GetGameTime()
	local utcHour = tonumber(date("!%H"))
	local utcMinute = tonumber(date("!%M"))
	local ser = serverHour + serverMinute / 60
	local utc = utcHour + utcMinute / 60
	self.serverOffset = floor((ser - utc) * 2 + 0.5) / 2
	if self.serverOffset >= 12 then
		self.serverOffset = self.serverOffset - 24
	elseif self.serverOffset < -12 then
		self.serverOffset = self.serverOffset + 24
	end
	return self.serverOffset
end

function MG:TimeDifference(dateA, dateB)
	local dateC = {}
	dateC.year = dateA.year - dateB.year
	dateC.month = dateA.month - dateB.month
	dateC.day = dateA.day - dateB.day
	dateC.hour = dateA.hour - dateB.hour
	dateC.min = dateA.min - dateB.min
	dateC.sec = dateA.sec - dateB.sec
	return self:ConformDate(dateC)
end

function MG:TimeProduct(dateA, dateB)
	local dateC = {}
	dateC.year = dateA.year + dateB.year
	dateC.month = dateA.month + dateB.month
	dateC.day = dateA.day + dateB.day
	dateC.hour = dateA.hour + dateB.hour
	dateC.min = dateA.min + dateB.min  
	dateC.sec = dateA.sec + dateB.sec	
	return self:ConformDate(dateC)
end

function MG:ConformDate(date)
	--self:Print("ConformDate: "..self:Date2String(date))
		
	--self:DisplayDateTable(date)
		
	local daysInYear = {[0] = 365, [1] = 366}
	local daysInMonth = {[0] = 0, [1] = 31, [2] = 29, [3] = 31, [4] = 30, [5] = 31, [6] = 30, [7] = 31, [8] = 31, [9] = 30, [10] = 31, [11] = 30, [12] = 31}
	--Seconds
	date.min = date.min + floor(date.sec / 60.0)
	date.sec = floor(date.sec % 60.0)
	if date.sec < 0 then
		date.sec = date.sec + 60
		date.min = date.min - 1
	end
	
	--Minutes
	date.hour = date.hour + floor(date.min / 60.0)
	date.min = floor(date.min % 60.0)
	if date.min < 0 then
		date.min = date.min + 60
		date.hour = date.hour - 1
	end
	
	--Hours
	date.day = date.day + floor(date.hour / 24.0)
	date.hour = floor(date.hour % 24.0)
	if date.hour < 0 then
		date.hour = date.hour + 24
		date.day = date.day - 1
	end
	
	--Months Phase 1
	date.year = date.year + floor(date.month / 12.0)
	date.month = floor(date.month % 12.0)
	if date.month < 0 then
		date.month = date.month + 12
		date.year = date.year - 1
	end
	
	--Days Phase 1
	if date.day >= 0 then
		catch = 0
		while (date.day > daysInYear[self:IsLeapYear(date.year)] and catch < 40) do 
			date.day = date.day - daysInYear[self:isLeapYear(date.year)]
			date.year = date.year + 1
			catch = catch + 1
		end
	else
		--Subtracting days
		catch = 0
		while(date.day < (daysInYear[self:IsLeapYear(date.year)]*-1) and catch < 40) do 
			date.day = date.day + daysInYear[self:isLeapYear(date.year)]
			date.year = date.year - 1
			catch = catch + 1
		end
	end
	
	--Month Phase 2
	local catch = 0
	if date.day >= 0 then
		catch = 0
--		self:Print("Day: "..date.day)
--		self:Print("Month: "..date.month)
--		self:Print("Days In Month: "..daysInMonth[0])
		while(date.day > daysInMonth[date.month] and catch < 40) do
			date.day = date.day - daysInMonth[date.month]
			date.month = date.month + 1
			if date.month > 12 then
				date.month = 12
				date.year = date.year - 1
			end
			catch = catch + 1
		end
	else
		--Subtracting days
		catch = 0
		while(date.day < (daysInYear[self:IsLeapYear(date.year)] * -1) and catch < 40) do 
			date.day = date.day + daysInYear[self:IsLeapYear(date.year)]
			date.month = date.month - 1
			if date.month < 1 then 
				date.month = 12
				date.year = date.year - 1
			end
			catch = catch + 1
		end
	end
	--self:Print("FinalDate: "..self:Date2String(date))
	return date
end

function MG:GetLastTimeOnline(t, status)
	local lastOnDate = self:TimeSinceLastTimeCode(t)
	return self:displayTimeSinceLastOn(lastOnDate, status)
end

function MG:DisplayDateTable(d)
	self:Print(d.year.."-"..d.month.."-"..d.day.."   "..d.hour..":"..d.min..":"..d.sec)
end


function MG:displayTimeSinceLastOn(d, status)
	--d = date("*t", timeCode)
	
	
	--self:DisplayDateTable(d)	
	
	
	local a = "Seconds"
	if status == 1 or status == 2 or status == 3 then
		return self:GetStatusText(status)
	else
		if d.sec == 1 then
			a = d.sec.." "..L["Time/Second"]
		end

		if d.sec > 1 then
			a = d.sec.." "..L["Time/Seconds"]
		end

		if d.min == 1 then
			a = d.min.." "..L["Time/Minute"]
		end

		if d.min > 1 then
			a = d.min.." "..L["Time/Minutes"]
		end

		if d.hour == 1 then
			a = d.hour.." "..L["Time/Hour"]
		end

		if d.hour > 1 then
			a = d.hour.." "..L["Time/Hours"]
		end

		if d.day == 1 then
			a = d.day.." "..L["Time/Day"]
		end

		if d.day > 1 then
			a = d.day.." "..L["Time/Days"]
		end

		if d.month == 1 then
			a = d.month.." "..L["Time/Month"]
		end

		if d.month > 1 then
			a = d.month.." "..L["Time/Months"]
		end

		if d.year == 1 then
			a = d.year.." "..L["Time/Year"]
		end

		if d.year > 1 then
			a = d.year.." "..L["Time/Years"]
		end
		
		return a
	end
end


-- Rank System
function MG:GetRankName(index)
	if type(index) == "number" and index >= 0 and index <= 3 then
		local rankName = {[0] = L["Recruit"], [1] = L["Member"], [2] = L["Officer"], [3] = L["GuildMaster"]}
		return rankName[index]
	else
		if type(index) ~= "number" then
			self:Print("Index is not a number: "..type(index))
		end
		self:Print("Index is wrong: "..index)
		return "ERROR"
	end
end

--Status System
function MG:GetStatusText(status)
	if status ~= nil then
		status = tonumber(status)
		local statusText = {[0]="Offline", [1]="Online", [2]="AFK", [3]="DND", [4]="U"}
		return statusText[status]
	else
		return "Unknown"
	end
end

--Message Template
function MG:CreateMsgTemplate(dataFieldCount)
	return string.rep(specialChar .. "%s", dataFieldCount) .. specialChar;
end


--function MG:GetFormatString(formatType)	
--	if formatType == "JOIN" then
--		return format(self:CreateMsgTemplate(7), "JOIN", "(.+)", "(%a+)", "(.+)", "(%d%d?)", "(.+)", "(.*)");
--	end
--end


--function MG:GetOnlinePlayers()
--	return self.db.realm.onlinePlayers
--end

function MetaGuildTextEditFrame_Show(editType)
	local frame = getglobal("MetaGuildTextEditFrame")
	
	
	if ( editType == "motd" ) then
		MetaGuildTextEditFrame:SetHeight(162);
		MetaGuildTextEditBox:SetMaxLetters(128);
		--MetaGuildTextEditBox:SetText(GetGuildRosterMOTD());
		MetaGuildTextEditFrameTitle:SetText("*MotD*");
	elseif ( editType == "info" ) then
		MetaGuildTextEditFrame:SetHeight(295);
		MetaGuildTextEditBox:SetMaxLetters(500);
		--MetaGuildTextEditBox:SetText(GuildInfoFrame.cachedInfoText);
		MetaGuildTextEditFrameTitle:SetText("*Info*");
	elseif ( editType == "mission" ) then
		MetaGuildTextEditFrame:SetHeight(295);
		MetaGuildTextEditBox:SetMaxLetters(500);
		--MetaGuildTextEditBox:SetText(self.db.realm.mission.text);
		MetaGuildTextEditFrameTitle:SetText("*Mission*");
		
	end
	--MetaGuildTextEditBox:SetScript("OnEnterPressed", MetaGuildTextEditFrame_OnAccept);
	MetaGuildTextEditFrame.type = editType;
	if self.db.realm[editType].text == nil then
		self.db.realm[editType].text = ""
	end
	MetaGuildTextEditBox:SetText(self.db.realm[editType].text);
	
	if MetaGuildTextEditFrame:IsShown() then
		MetaGuildTextEditFrame:Hide()
	else
		MetaGuildTextEditFrame:Show()
	end
	
	--FUBAR - add in author and date.
	
	MetaGuildTextEditBox:SetCursorPosition(0);
	MetaGuildTextEditBox:SetFocus();
	PlaySound("igMainMenuOptionCheckBoxOn");
end

function MetaGuildTextEditFrame_OnAccept()
	local infoText = MetaGuildTextEditBox:GetText()
	self:SetGuildInfoText(infoText, MetaGuildTextEditFrame.type)
	MetaGuildTextEditFrame:Hide();
end

function MG:SetGuildInfoText(infoText, editType)
	self.db.realm[editType].text = infoText
	self.db.realm[editType].time = self:GetTimeCode()
	self.db.realm[editType].author = self.db.realm.main
	if editType == "mission" then
		MetaGuildGuild_Update()
	end
end



